#include <Arduino.h>
#include "xtensa/core-macros.h"
#include <ESP32-HUB75-MatrixPanel-I2S-DMA.h>

#include "main.h"

#define R1 42
#define G1 40
#define BL1 41
#define R2 38
#define G2 37
#define BL2 39
#define CH_A 45
#define CH_B 36
#define CH_C 48
#define CH_D 35
#define CH_E 21
#define CLK 2
#define LAT 47
#define OE 14

#define PIN_E 21
#define PANEL_WIDTH 64
#define PANEL_HEIGHT 32         // Panel height of 64 will required PIN_E to be defined.

#define PANELS_NUMBER 1

#define PANE_WIDTH PANEL_WIDTH * PANELS_NUMBER
#define PANE_HEIGHT PANEL_HEIGHT
#define NUM_LEDS PANE_WIDTH*PANE_HEIGHT

#define ONBOARD_LED 13

MatrixPanel_I2S_DMA *matrix = nullptr;

int start_x = 5;
int start_y = 5;
int end_x = 59;
int end_y = 27;

int anchor_x = 16;
int anchor_y = 24;

#define BLOB_POINTS 32
#define BLOB_RADIUS 12
typedef struct {
  int x_coords[BLOB_POINTS];
  int y_coords[BLOB_POINTS];
} Blob;

Blob blob;

void setup(){
  Serial.begin(BAUD_RATE);
  pinMode(ONBOARD_LED, OUTPUT);

  // redefine pins if required
  HUB75_I2S_CFG::i2s_pins _pins={R1, G1, BL1, R2, G2, BL2, CH_A, CH_B, CH_C, CH_D, CH_E, LAT, OE, CLK};
  HUB75_I2S_CFG mxconfig(PANEL_WIDTH, PANEL_HEIGHT, PANELS_NUMBER, _pins);

  mxconfig.gpio.e = PIN_E;
  mxconfig.driver = HUB75_I2S_CFG::FM6126A;   // for panels using FM6126A chips

  mxconfig.latch_blanking = 4;
  mxconfig.i2sspeed = HUB75_I2S_CFG::HZ_10M;
  mxconfig.clkphase = false;

  mxconfig.double_buff = true;

  matrix = new MatrixPanel_I2S_DMA(mxconfig);
  matrix->begin();
  matrix->setBrightness8(64);
  matrix->fillScreenRGB888(0, 0, 0);

  for(int i = 0; i < BLOB_POINTS; i++) {
    //float angle = (360 / BLOB_POINTS) * (i+1);
    float angle = ((PI * 2) / BLOB_POINTS) * (i+1);
    blob.x_coords[i] = random(-6, 7) + 31 + round(BLOB_RADIUS * sin(angle));
    blob.y_coords[i] = random(-2, 7) + 15 + round(BLOB_RADIUS * cos(angle));
    //blob.x_coords[i] = random(0, 64);
    //blob.y_coords[i] = random(0, 32);
  }
}

int interpolate(int from, int to, float percent) {
    int difference = to - from;
    return from + ( difference * percent );
}

void draw_bezier(int x1, int y1, int x2, int y2, int x3, int y3) {
  // draw lines to p2
  //matrix->drawLine(x1, y1, x2, y2, matrix->color565(16, 16, 16));
  //matrix->drawLine(x2, y2, x3, y3, matrix->color565(16, 16, 16));

  for (float percent = 0; percent < 1; percent += 0.01) {
    int xa = interpolate(x1, x2, percent);
    int ya = interpolate(y1, y2, percent);
    int xb = interpolate(x2, x3, percent);
    int yb = interpolate(y2, y3, percent);
    //matrix->drawLine(xa, ya, xb, yb, matrix->color565(0, 128, 0));

    int x = interpolate(xa, xb, percent);
    int y = interpolate(ya, yb, percent);

    matrix->drawPixel(x, y, matrix->color565(255, 80, 83));
  }
}



void loop() {
  matrix ->flipDMABuffer();
  //matrix->clearScreen();
  matrix->fillScreenRGB888(15, 0, 10);
  // draw grid

  int h_lines = PANEL_HEIGHT / 7;
  int v_lines = PANEL_WIDTH / 7;
  for(int i = 0; i < PANEL_WIDTH; i++) {
    if(i % 7 == 2) {
      matrix->drawFastVLine(i, 0, PANEL_HEIGHT, matrix->color565(41, 17, 76));
    }
  }
  for(int i = 0; i <= h_lines; i++) {
    matrix->drawFastHLine(0, 7*i + 1, PANEL_WIDTH, matrix->color565(41, 17, 76));
  }

  for(int i = 0; i < BLOB_POINTS; i+=2) {
    int end_offset = i + 2;
    if ( end_offset >= BLOB_POINTS) {
      end_offset = 0;
    }
    draw_bezier(blob.x_coords[i],
        blob.y_coords[i],
        blob.x_coords[i+1],
        blob.y_coords[i+1],
        blob.x_coords[end_offset],
        blob.y_coords[end_offset]
    );
    //matrix->drawPixel(blob.x_coords[i], blob.y_coords[i], matrix->color565(128, 128, 128));
    blob.x_coords[i+1] += random(-1, 2);
    if (blob.x_coords[i+1] >= PANEL_WIDTH) {
      blob.x_coords[i+1] = PANEL_WIDTH - 1;
    }
  if (blob.x_coords[i+1] < 0) {
      blob.x_coords[i+1] = 0;
    }
    //blob.y_coords[i+1] += random(-1, 2);
    //matrix->drawPixel(blob.x_coords[i], blob.y_coords[i], matrix->color565(0, 0, 128));
    //matrix->drawPixel(blob.x_coords[i+1], blob.y_coords[i+1], matrix->color565(128, 0, 0));
    //matrix->drawPixel(blob.x_coords[end_offset], blob.y_coords[end_offset], matrix->color565(0, 0, 128));
  }
  delay(500);
}

